//==============================================================================
// VERSION 20051123 (c) Dipl. Inform. H. Gorski [www.HeikoGorski.de]
//==============================================================================
#include "stdafx.h"

#include <iostream>
#include <iomanip>
#include <ios>
#include <math.h>
#include <locale.h>
#include <string>
#include <strstream>
#include "lfout.h"

using namespace std;

#define LFOUT_NOTUSED -1
#define HG_VISUAL_C_2005

//------------------------------------------------------------------------------
// Constructor
// normale Formatierung, tut nix - nur der Vollstndigkeit halber
// -1 hat Bedeutung von "NICHT GESETZT", da NULL ein gltiger Wert ist
//------------------------------------------------------------------------------
lfout::lfout (long double pValue)
{
  set (pValue, LFOUT_NOTUSED, LFOUT_NOTUSED);
}

//------------------------------------------------------------------------------
// Constructor
// nur Nachkommastellen angegeben
// -1 hat Bedeutung von "NICHT GESETZT", da NULL ein gltiger Wert ist
//------------------------------------------------------------------------------
lfout::lfout (long double pValue, int pNachkomma)
{
  set (pValue, LFOUT_NOTUSED,  pNachkomma);
}

//------------------------------------------------------------------------------
// Constructor
// Vor- und Nachkommastellen angegeben
//------------------------------------------------------------------------------
lfout::lfout (long double pValue, int pGesamtstellen, int pNachkomma)
{
  set (pValue, pGesamtstellen, pNachkomma);
}

//------------------------------------------------------------------------------
// Setzen der Klassenvariablen gem der an die Klasse LFOUT bergebenen Werte
//------------------------------------------------------------------------------
lfout& lfout::set (long double pValue, int pGesamtstellen, int pNachkomma)
{
  //----------------------------------------------------------------------------
  // Setzen der LONG DOUBLE-Variablen auf den im Parameter bergebenen Wert
  //----------------------------------------------------------------------------
  fValue = pValue;

  //----------------------------------------------------------------------------
  // Setzen der Nachkomma-Variable, Korrektur illegaler Werte (kleiner -1)
  // -1 bedeutet Standardwert (d.h. 6 Nachkommastellen)
  //  0 bedeutet keine Nachkommastellen und ist daher ein gltiger Wert
  // Da damit die Anzahl der Nachkommastellen feststeht, kann diese in NK
  // gespeichert werden
  //----------------------------------------------------------------------------
  if (pNachkomma < 0)
  {
    pNachkomma = LFOUT_NOTUSED;
  }

  nNachkomma = pNachkomma;

  //----------------------------------------------------------------------------
  // Setzen der Vorkomma-Variable, Korrektur illegaler Werte (kleiner -1)
  // -1 bedeutet Standardwert (d.h. die bentigte Anzahl von Vorkommastellen)
  // die Anzahl der Vorkommastellen ist nur dann von Interesse, wenn die
  // Anzahl der bentigten Stellen KLEINER ist als die Anzahl der gewnschten
  // Stellen.
  //----------------------------------------------------------------------------
  if (pGesamtstellen  < 0)
  {
    pGesamtstellen = LFOUT_NOTUSED;
  }

  //----------------------------------------------------------------------------
  // Wenn Nachkommastellen gewnscht sind und die angegebene Anzahl GRSSER
  // ist als die GESAMTZAHL der gewnschten Stellen also z.B. lfout(x,3,9)
  // und die Gesamtanzahl der Stellen wurde bergeben (ungleich -1)
  // dann immer die bentigte Anzahl an Vorkommastellen ausgeben (wie -1)
  //----------------------------------------------------------------------------
  if (pNachkomma > 0)
  {
    if ((pNachkomma >= pGesamtstellen) && (pGesamtstellen != LFOUT_NOTUSED))
    {
      pGesamtstellen = LFOUT_NOTUSED;
    }
  }

  //----------------------------------------------------------------------------
  // Wenn Nachkommastellen gewnscht sind und die angegebene Gesamtanzahl an
  // Stellen wurde bergeben (ungleich -1), dann ist die Anzahl der Vorkomma-
  // stellen zu berechnen. Sonst entspricht die Anzahl der Vorkommastellen
  // der Anzahl der Gesamtstellen
  //----------------------------------------------------------------------------
  if ((pNachkomma > 0) && (pGesamtstellen != LFOUT_NOTUSED))
  {
    nVorkomma = pGesamtstellen - pNachkomma - 1;
  }
  else
  {
    nVorkomma = pGesamtstellen;
  }
  return *this;
}

//------------------------------------------------------------------------------
// Formatierte Ausgabe eines LFOUT-Objekts in einem OSTREAM
//------------------------------------------------------------------------------
ostream& operator<< (ostream& pOutstream, const lfout& pLfout)
{
  //----------------------------------------------------------------------------
  // Wert der Vorkommastellen werden durch einfaches Abschneiden der Nach-
  // kommastellen gebildet
  // Nachkommastellen erstmal auf Null setzen
  //----------------------------------------------------------------------------
  long double fRundung       = 0.0;
  long double fTempvorkomma  = floor (pLfout.fValue);
  long double fTempnachkomma = 0.0;
  lconv      *zLandSpezifika = localeconv();
  char       aString[5000]   = "";

  //----------------------------------------------------------------------------
  // Keine Exponentialausgaben zulassen! Dazu mu ein Flag gesetzt werden,
  // der vorherige Wert des Flags wird zwischengespeichert und vor Funktions-
  // ende restauriert
  //----------------------------------------------------------------------------
  long nOldflags = pOutstream.setf(ios::fixed, ios::floatfield);
  bool bLeft     = nOldflags | ios::left;
  bool bRight    = nOldflags | ios::right;
 
  //----------------------------------------------------------------------------
  // Gleiches gilt fr die Ausgabeprzision, die hier auf Null (keine Nach-
  // kommastellen) gesetzt wird, die Nachkommastellen machen wir selbst.
  //----------------------------------------------------------------------------
  int nOldPrecision = pOutstream.precision(0);

  //----------------------------------------------------------------------------
  // Wenn definierte Anzahl von Nachkommastellen gewnscht ist, dann diese
  // in Vorkommastellen einer anderen Variablen umwandeln
  // Die Addition von +0.1 vermeidet Rundungsfehler bei der Subtraktion
  // ist der Wert fr Nachkommastellen gleich -1, dann 6 Stellen ausgeben
  //----------------------------------------------------------------------------
  if (pLfout.nNachkomma != LFOUT_NOTUSED)
  {
    fTempnachkomma = ((pLfout.fValue - fTempvorkomma) * (pow ((long double)10.0, pLfout.nNachkomma)));
  }
  else
  {
    fTempnachkomma = ((pLfout.fValue - fTempvorkomma) * (pow ((long double)10.0, 6)));
  }
  fRundung       = fTempnachkomma - floor (fTempnachkomma);
  fTempnachkomma = floor (fTempnachkomma);

  if (fRundung >= 0.5)
  {
    fTempnachkomma += 1.0;
  }

  double fOverflow = (fTempnachkomma / pow((long double)10.0, pLfout.nNachkomma));
  if (fOverflow == 1.0)
  {
    fTempvorkomma  += 1.0;
    fTempnachkomma  = 0.0;
  }

  //----------------------------------------------------------------------------
  // gibt es weder fr Vor- noch fr Nachkommastellen eine Vorgabe, dann die
  // Flags wiederherstellen und ganz normal ausgeben
  //----------------------------------------------------------------------------
  if ((pLfout.nNachkomma == LFOUT_NOTUSED) && (pLfout.nVorkomma == LFOUT_NOTUSED))
  {
    pOutstream.precision (nOldPrecision);
    pOutstream.setf      (nOldflags, std::ios::floatfield);
    pOutstream << pLfout.fValue;
    return (pOutstream);
  }

  //----------------------------------------------------------------------------
  // Sonst jetzt einen temporren String mit dem gewnschten Format aufbauen
  //----------------------------------------------------------------------------
  strstream sOutput (aString, 5000);
  sOutput << fTempvorkomma;

  //----------------------------------------------------------------------------
  // Ausgabe der Nachkommastellen gem Anzahl der gewnschten Stellen oder
  // die bentigten Stellen (ELSE-Fall)
  // Dazu mu man mit Nullen auffllen!
  //----------------------------------------------------------------------------
  if (pLfout.nNachkomma > 0)
  {
    char cOldFill = sOutput.fill ('0');
    sOutput << zLandSpezifika->decimal_point
            << std::setw(pLfout.nNachkomma) << fTempnachkomma;
    sOutput.fill (cOldFill);
  }

  if (pLfout.nNachkomma < 0)
  {
    sOutput << zLandSpezifika->decimal_point << fTempnachkomma;
  }

  pOutstream << setw (pLfout.nNachkomma + pLfout.nVorkomma + 1) << aString; 
 
  //----------------------------------------------------------------------------
  // Flags wiederherstellen
  //----------------------------------------------------------------------------
  pOutstream.precision (nOldPrecision);
  pOutstream.setf      (nOldflags, std::ios::floatfield);

  return (pOutstream);
}

